"""
    CLI for usage
"""
import cProfile
import logging
import pstats
import sys
from pathlib import Path

import rich_click as click
from bx_py_utils.path import assert_is_file
from rich import print  # noqa
from rich_click import RichGroup

import MC6809
from MC6809 import constants
from MC6809.core.bechmark import run_benchmark


logger = logging.getLogger(__name__)


PACKAGE_ROOT = Path(MC6809.__file__).parent.parent
assert_is_file(PACKAGE_ROOT / 'pyproject.toml')

OPTION_ARGS_DEFAULT_TRUE = dict(is_flag=True, show_default=True, default=True)
OPTION_ARGS_DEFAULT_FALSE = dict(is_flag=True, show_default=True, default=False)
ARGUMENT_EXISTING_DIR = dict(
    type=click.Path(exists=True, file_okay=False, dir_okay=True, readable=True, path_type=Path)
)
ARGUMENT_NOT_EXISTING_DIR = dict(
    type=click.Path(
        exists=False,
        file_okay=False,
        dir_okay=True,
        readable=False,
        writable=True,
        path_type=Path,
    )
)
ARGUMENT_EXISTING_FILE = dict(
    type=click.Path(exists=True, file_okay=True, dir_okay=False, readable=True, path_type=Path)
)


class ClickGroup(RichGroup):  # FIXME: How to set the "info_name" easier?
    def make_context(self, info_name, *args, **kwargs):
        info_name = './cli.py'
        return super().make_context(info_name, *args, **kwargs)


@click.group(
    cls=ClickGroup,
    epilog=constants.CLI_EPILOG,
)
def cli():
    pass


@click.command()
def version():
    """Print version and exit"""
    # Pseudo command, because the version always printed on every CLI call ;)
    sys.exit(0)


cli.add_command(version)


@click.command()
@click.option('--loops', type=int, default=6, show_default=True, help='How many benchmark loops should be run?')
@click.option('--multiply', type=int, default=15, show_default=True, help='est data multiplier')
def benchmark(loops, multiply):
    """
    Run a MC6809 emulation benchmark
    """
    run_benchmark(loops=loops, multiply=multiply)


cli.add_command(benchmark)


@click.command()
@click.option('--loops', type=int, default=6, show_default=True, help='How many benchmark loops should be run?')
@click.option('--multiply', type=int, default=15, show_default=True, help='est data multiplier')
def profile(loops, multiply):
    """
    Profile the MC6809 emulation benchmark
    """
    with cProfile.Profile() as pr:
        run_benchmark(loops=loops, multiply=multiply)
    pstats.Stats(pr).sort_stats('tottime', 'cumulative', 'calls').print_stats(20)


cli.add_command(profile)


def main():
    print(f'[bold][green]MC6809[/green] v[cyan]{MC6809.__version__}')

    # Execute Click CLI:
    cli.name = './cli.py'
    cli()



from bx_py_utils.auto_doc import assert_readme_block
from bx_py_utils.path import assert_is_file
from manageprojects.test_utils.click_cli_utils import invoke_click
from manageprojects.tests.base import BaseTestCase

from MC6809 import constants
from MC6809.cli.cli_app import PACKAGE_ROOT, cli
from MC6809.cli.dev import cli as dev_cli


def assert_cli_help_in_readme(text_block: str, marker: str):
    README_PATH = PACKAGE_ROOT / 'README.md'
    assert_is_file(README_PATH)

    text_block = text_block.replace(constants.CLI_EPILOG, '')
    text_block = f'```\n{text_block.strip()}\n```'
    assert_readme_block(
        readme_path=README_PATH,
        text_block=text_block,
        start_marker_line=f'[comment]: <> (✂✂✂ auto generated {marker} start ✂✂✂)',
        end_marker_line=f'[comment]: <> (✂✂✂ auto generated {marker} end ✂✂✂)',
    )


class ReadmeTestCase(BaseTestCase):
    def test_main_help(self):
        stdout = invoke_click(cli, '--help')
        self.assert_in_content(
            got=stdout,
            parts=(
                'Usage: ./cli.py [OPTIONS] COMMAND [ARGS]...',
                ' benchmark ',
                ' profile ',
                constants.CLI_EPILOG,
            ),
        )
        assert_cli_help_in_readme(text_block=stdout, marker='main help')

    def test_dev_cli_main_help(self):
        stdout = invoke_click(dev_cli, '--help')
        self.assert_in_content(
            got=stdout,
            parts=(
                'Usage: ./dev-cli.py [OPTIONS] COMMAND [ARGS]...',
                ' check-code-style ',
                ' coverage ',
                constants.CLI_EPILOG,
            ),
        )
        assert_cli_help_in_readme(text_block=stdout, marker='dev help')

    def test_benchmark_help(self):
        stdout = invoke_click(cli, 'benchmark', '--help')
        self.assert_in_content(
            got=stdout,
            parts=(
                'Usage: ./cli.py benchmark [OPTIONS]',
                ' --loops ',
                ' --multiply ',
            ),
        )
        assert_cli_help_in_readme(text_block=stdout, marker='benchmark help')

    def test_profile_help(self):
        stdout = invoke_click(cli, 'profile', '--help')
        self.assert_in_content(
            got=stdout,
            parts=(
                'Usage: ./cli.py profile [OPTIONS]',
                ' --loops ',
                ' --multiply ',
            ),
        )
        assert_cli_help_in_readme(text_block=stdout, marker='profile help')
